<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 279 $
 * $Id: UserDefaultFactory.class.php 279 2006-10-24 21:23:09Z ipso $
 * $Date: 2006-10-24 14:23:09 -0700 (Tue, 24 Oct 2006) $
 */
class UserDefaultFactory extends Factory {
	protected $table = 'user_default';
	protected $pk_sequence_name = 'user_default_id_seq'; //PK Sequence name

	protected $company_obj = NULL;
	protected $title_obj = NULL;

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $payperiodschedule_validator_msg = 'Pay Period schedule is invalid';
	protected $payperiodschedule_validator_label = 'pay_period_schedule_id';

	protected $policygroup_validator_msg = 'Policy Group is invalid';
	protected $policygroup_validator_label = 'policy_group_id';

	protected $title_validator_msg = 'Title is invalid';
	protected $title_validator_label = 'title';

	protected $city_validator_regex = '/^[a-zA-Z0-9-\.\ ]{1,250}$/i';
	protected $city_validator_regex_msg = 'City contains invalid characters';
	protected $city_validator_length_msg = 'City name is too short or too long';
	protected $city_validator_label = 'city';

	protected $province_validator_options_msg = 'Invalid Province';
	protected $province_validator_label = 'province';

	protected $country_validator_options_msg = 'Invalid Country';
	protected $country_validator_label = 'country';

	protected $workphone_validator_isphonenumber_msg = 'Work phone number is invalid';
	protected $workphone_validator_label = 'work_phone';

	protected $workphoneext_validator_length_msg = 'Work phone number extension is too short or too long';
	protected $workphoneext_validator_label = 'work_phone_ext';

	protected $workemail_validator_isemail_msg = 'Work Email address is invalid';
	protected $workemail_validator_label = 'work_email';

	protected $hiredate_validator_isdate_msg = 'Hire date is invalid';
	protected $hiredate_validator_label = 'hire_date';

	protected $defaultbranch_validator_msg = 'Invalid Default Branch';
	protected $defaultbranch_validator_label = 'default_branch';

	protected $defaultdepartment_validator_msg = 'Invalid Default Department';
	protected $defaultdepartment_validator_label = 'default_department';

	protected $employeenumber_validator_length_msg = 'Employee number is too short or too long';
	protected $employeenumber_validator_label = 'employee_number';

	protected $date_format_validator_options_msg = 'Incorrect date format';
	protected $date_format_validator_label = 'date_format';

	protected $time_format_validator_options_msg = 'Incorrect time format';
	protected $time_format_validator_label = 'time_format';

	protected $time_zone_validator_options_msg = 'Incorrect time zone';
	protected $time_zone_validator_label = 'time_zone';

	protected $time_unit_format_validator_options_msg = 'Incorrect time units';
	protected $time_unit_format_validator_label = 'time_unit_format';

	protected $items_per_page_validator_value_msg = 'Items per page must be between 10 and 200';
	protected $items_per_page_validator_label = 'items_per_page';

	protected $start_week_day_validator_options_msg = 'Incorrect day to start a week on';
	protected $start_week_day_validator_label = 'start_week_day';

	protected $companydeduction_validator_msg = 'Deduction is invalid';
	protected $companydeduction_validator_label = 'company_deduction';

	function getCompanyObject() {
		if ( is_object($this->company_obj) ) {
			return $this->company_obj;
		} else {
			$clf = new CompanyListFactory();
			$this->company_obj = $clf->getById( $this->getCompany() )->getCurrent();

			return $this->company_obj;
		}
	}

	function getTitleObject() {
		if ( is_object($this->title_obj) ) {
			return $this->title_obj;
		} else {

			$utlf = new UserTitleListFactory();
			$utlf->getById( $this->getTitle() );

			if ( $utlf->getRecordCount() == 1 ) {
				$this->title_obj = $utlf->getCurrent();

				return $this->title_obj;
			}

			return FALSE;
		}
	}

	function getCompany() {
		if ( isset($this->data['company_id']) ) {
			return $this->data['company_id'];
		}

		return FALSE;
	}
	function setCompany($id) {
		$id = trim($id);

		Debug::Text('Company ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getPayPeriodSchedule() {
		if ( isset($this->data['pay_period_schedule_id']) ) {
			return $this->data['pay_period_schedule_id'];
		}

		return FALSE;
	}
	function setPayPeriodSchedule($id) {
		$id = trim($id);

		$ppslf = new PayPeriodScheduleListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->payperiodschedule_validator_label,
															$ppslf->getByID($id),
															$this->payperiodschedule_validator_msg
															) ) {
			$this->data['pay_period_schedule_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getPolicyGroup() {
		if ( isset($this->data['policy_group_id']) ) {
			return $this->data['policy_group_id'];
		}

		return FALSE;
	}
	function setPolicyGroup($id) {
		$id = trim($id);

		$pglf = new PolicyGroupListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->policygroup_validator_label,
															$pglf->getByID($id),
															$this->policygroup_validator_msg
															) ) {
			$this->data['policy_group_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getEmployeeNumber() {
		if ( isset($this->data['employee_number']) ) {
			return $this->data['employee_number'];
		}

		return FALSE;
	}
	function setEmployeeNumber($value) {
		$value = trim($value);

		if 	(
				$value == ''
				OR
					$this->Validator->isLength(		$this->employeenumber_validator_label,
													$value,
													$this->employeenumber_validator_length_msg,
													1,
													100) ) {

			$this->data['employee_number'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getTitle() {
		if ( isset($this->data['title_id']) ) {
			return $this->data['title_id'];
		}

		return FALSE;
	}
	function setTitle($id) {
		$id = trim($id);

		Debug::Text('Title ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$utlf = new UserTitleListFactory();

		if (
				$id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->title_validator_label,
														$utlf->getByID($id),
														$this->title_validator_msg
													) ) {

			$this->data['title_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getDefaultBranch() {
		if ( isset($this->data['default_branch_id']) ) {
			return $this->data['default_branch_id'];
		}

		return FALSE;
	}
	function setDefaultBranch($id) {
		$id = trim($id);

		Debug::Text('Branch ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$blf = new BranchListFactory();

		if (
				$id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->defaultbranch_validator_label,
														$blf->getByID($id),
														$this->defaultbranch_validator_msg
													) ) {

			$this->data['default_branch_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getDefaultDepartment() {
		if ( isset($this->data['default_department_id']) ) {
			return $this->data['default_department_id'];
		}

		return FALSE;
	}
	function setDefaultDepartment($id) {
		$id = trim($id);

		Debug::Text('Department ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$dlf = new DepartmentListFactory();

		if (
				$id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->defaultdepartment_validator_label,
														$dlf->getByID($id),
														$this->defaultdepartment_validator_msg
													) ) {

			$this->data['default_department_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getCity() {
		if ( isset($this->data['city']) ) {
			return $this->data['city'];
		}

		return FALSE;
	}
	function setCity($city) {
		$city = trim($city);

		if 	(
				$city == ''
				OR
				(
				$this->Validator->isRegEx(		$this->city_validator_label,
												$city,
												$this->city_validator_regex_msg,
												$this->city_validator_regex)
				AND
					$this->Validator->isLength(		$this->city_validator_label,
													$city,
													$this->city_validator_length_msg,
													2,
													250)
				)
				) {

			$this->data['city'] = $city;

			return TRUE;
		}

		return FALSE;
	}

	function getProvince() {
		if ( isset($this->data['province']) ) {
			return $this->data['province'];
		}

		return FALSE;
	}
	function setProvince($province) {
		$province = trim($province);

		Debug::Text('Country: '. $this->getCountry() .' Province: '. $province, __FILE__, __LINE__, __METHOD__,10);

		$cf = new CompanyFactory();

		$options_arr = $cf->getOptions('province');
		if ( isset($options_arr[$this->getCountry()]) ) {
			$options = $options_arr[$this->getCountry()];
		} else {
			$options = array();
		}

		if ( $this->Validator->inArrayKey(		$this->province_validator_label,
												$province,
												$this->province_validator_options_msg,
												$options ) ) {

			$this->data['province'] = $province;

			return TRUE;
		}

		return FALSE;
	}

	function getCountry() {
		if ( isset($this->data['country']) ) {
			return $this->data['country'];
		}

		return FALSE;
	}
	function setCountry($country) {
		$country = trim($country);

		$cf = new CompanyFactory();

		if ( $this->Validator->inArrayKey(		$this->country_validator_label,
												$country,
												$this->country_validator_options_msg,
												$cf->getOptions('country') ) ) {

			$this->data['country'] = $country;

			return TRUE;
		}

		return FALSE;
	}

	function getWorkPhone() {
		if ( isset($this->data['work_phone']) ) {
			return $this->data['work_phone'];
		}

		return FALSE;
	}
	function setWorkPhone($work_phone) {
		$work_phone = trim($work_phone);

		if 	(
				$work_phone == ''
				OR
				$this->Validator->isPhoneNumber(		$this->workphone_validator_label,
														$work_phone,
														$this->workphone_validator_isphonenumber_msg) ) {

			$this->data['work_phone'] = $work_phone;

			return TRUE;
		}

		return FALSE;
	}

	function getWorkPhoneExt() {
		if ( isset($this->data['work_phone_ext']) ) {
			return $this->data['work_phone_ext'];
		}

		return FALSE;
	}
	function setWorkPhoneExt($work_phone_ext) {
		$work_phone_ext = $this->Validator->stripNonNumeric( trim($work_phone_ext) );

		if ( 	$work_phone_ext == ''
				OR $this->Validator->isLength(		$this->workphoneext_validator_label,
													$work_phone_ext,
													$this->workphoneext_validator_length_msg,
													2,
													10) ) {

			$this->data['work_phone_ext'] = $work_phone_ext;

			return TRUE;
		}

		return FALSE;

	}

	function getWorkEmail() {
		if ( isset($this->data['work_email']) ) {
			return $this->data['work_email'];
		}

		return FALSE;
	}
	function setWorkEmail($work_email) {
		$work_email = trim($work_email);

		if 	(	$work_email == ''
					OR	$this->Validator->isEmail(	$this->workemail_validator_label,
													$work_email,
													$this->workemail_validator_isemail_msg) ) {

			$this->data['work_email'] = $work_email;

			return TRUE;
		}

		return FALSE;
	}

	function getHireDate() {
		if ( isset($this->data['hire_date']) ) {
			return $this->data['hire_date'];
		}

		return FALSE;
	}
	function setHireDate($epoch) {
		if ( empty($epoch) ) {
			$epoch = NULL;
		}

		if 	(	$epoch == ''
				OR
				$this->Validator->isDate(		$this->hiredate_validator_label,
												$epoch,
												$this->hiredate_validator_isdate_msg) ) {

			$this->data['hire_date'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}

	/*

		User Preferences

	*/
	function getDateFormat() {
		if ( isset($this->data['date_format']) ) {
			return $this->data['date_format'];
		}

		return FALSE;
	}
	function setDateFormat($date_format) {
		$date_format = trim($date_format);

		$upf = new UserPreferenceFactory();

		$key = Option::getByValue($date_format, $upf->getOptions('date_format') );
		if ($key !== FALSE) {
			$date_format = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->date_format_validator_label,
											$date_format,
											$this->date_format_validator_options_msg,
											$upf->getOptions('date_format')) ) {

			$this->data['date_format'] = $date_format;

			return FALSE;
		}

		return FALSE;
	}

	function getTimeFormat() {
		if ( isset($this->data['time_format']) ) {
			return $this->data['time_format'];
		}

		return FALSE;
	}
	function setTimeFormat($time_format) {
		$time_format = trim($time_format);

		$upf = new UserPreferenceFactory();

		$key = Option::getByValue($time_format, $upf->getOptions('time_format') );
		if ($key !== FALSE) {
			$time_format = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->time_format_validator_label,
											$time_format,
											$this->time_format_validator_options_msg,
											$upf->getOptions('time_format')) ) {

			$this->data['time_format'] = $time_format;

			return FALSE;
		}

		return FALSE;
	}

	function getTimeZone() {
		if ( isset($this->data['time_zone']) ) {
			return $this->data['time_zone'];
		}

		return FALSE;
	}
	function setTimeZone($time_zone) {
		$time_zone = trim($time_zone);

		$upf = new UserPreferenceFactory();

		$key = Option::getByValue($time_zone, $upf->getOptions('time_zone') );
		if ($key !== FALSE) {
			$time_zone = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->time_zone_validator_label,
											$time_zone,
											$this->time_zone_validator_options_msg,
											$upf->getOptions('time_zone')) ) {

			$this->data['time_zone'] = $time_zone;

			return FALSE;
		}

		return FALSE;
	}

	function getTimeUnitFormatExample() {
		$options = $this->getOptions('time_unit_format');

		return $options[$this->getTimeUnitFormat()];
	}
	function getTimeUnitFormat() {
		return $this->data['time_unit_format'];
	}
	function setTimeUnitFormat($time_unit_format) {
		$time_unit_format = trim($time_unit_format);

		$upf = new UserPreferenceFactory();

		$key = Option::getByValue($time_unit_format, $upf->getOptions('time_unit_format') );
		if ($key !== FALSE) {
			$time_unit_format = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->time_unit_format_validator_label,
											$time_unit_format,
											$this->time_unit_format_validator_options_msg,
											$upf->getOptions('time_unit_format')) ) {

			$this->data['time_unit_format'] = $time_unit_format;

			return FALSE;
		}

		return FALSE;
	}

	function getItemsPerPage() {
		if ( isset($this->data['items_per_page']) ) {
			return $this->data['items_per_page'];
		}

		return FALSE;
	}
	function setItemsPerPage($items_per_page) {
		$items_per_page = trim($items_per_page);

		if 	($items_per_page != '' AND $items_per_page >= 1 AND $items_per_page <= 200) {

			$this->data['items_per_page'] = $items_per_page;

			return TRUE;
		} else {

			$this->Validator->isTrue(		$this->items_per_page_validator_label,
											FALSE,
											$this->items_per_page_validator_value_msg);
		}

		return FALSE;
	}

	function getStartWeekDay() {
		if ( isset($this->data['start_week_day']) ) {
			return $this->data['start_week_day'];
		}

		return FALSE;
	}
	function setStartWeekDay($value) {
		$value = trim($value);

		$upf = new UserPreferenceFactory();

		$key = Option::getByValue($value, $upf->getOptions('start_week_day') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->start_week_day_validator_label,
											$value,
											$this->start_week_day_validator_options_msg,
											$upf->getOptions('start_week_day')) ) {

			$this->data['start_week_day'] = $value;

			return FALSE;
		}

		return FALSE;
	}
	/*

		Company Deductions

	*/
	function getCompanyDeduction() {
		$udcdlf = new UserDefaultCompanyDeductionListFactory();
		$udcdlf->getByUserDefaultId( $this->getId() );
		foreach ($udcdlf as $obj) {
			$list[] = $obj->getCompanyDeduction();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setCompanyDeduction($ids) {
		Debug::text('Setting Company Deduction IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if ( is_array($ids) ) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$udcdlf = new UserDefaultCompanyDeductionListFactory();
				$udcdlf->getByUserDefaultId( $this->getId() );

				$tmp_ids = array();
				foreach ($udcdlf as $obj) {
					$id = $obj->getCompanyDeduction();
					Debug::text('ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			} else {
				$tmp_ids = array();
			}

			//Insert new mappings.
			//$lf = new UserListFactory();
			$cdlf = new CompanyDeductionListFactory();

			foreach ($ids as $id) {
				if ( $id != FALSE AND isset($ids) AND !in_array($id, $tmp_ids) ) {
					$udcdf = new UserDefaultCompanyDeductionFactory();
					$udcdf->setUserDefault( $this->getId() );
					$udcdf->setCompanyDeduction( $id );

					$obj = $cdlf->getById( $id )->getCurrent();

					if ($this->Validator->isTrue(		$this->companydeduction_validator_label,
														$udcdf->Validator->isValid(),
														$this->companydeduction_validator_msg.' ('. $obj->getName() .')' )) {
						$udcdf->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function Validate() {
		if ( $this->getCompany() == FALSE ) {
			$this->Validator->isTrue(		$this->company_validator_label,
											FALSE,
											$this->company_validator_msg);
		}

		return TRUE;
	}

	function postSave() {
		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, ' User Default Information', NULL, $this->getTable() );
	}

}
?>
