<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 649 $
 * $Id: WagesPayableSummary.php 649 2007-02-23 22:47:59Z ipso $
 * $Date: 2007-02-23 14:47:59 -0800 (Fri, 23 Feb 2007) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');

if ( !$permission->Check('report','enabled')
		OR !$permission->Check('report','view_wages_payable_summary') ) {
	$permission->Redirect( FALSE ); //Redirect
}

$title = 'Wages Payable Report';
$smarty->assign('title', $title);

//User Wage cache array.
function getUserWageObject( $user_wage_id ) {
	global $user_wage_obj;

	if ( isset($user_wage_obj[$user_wage_id])
		AND is_object($user_wage_obj[$user_wage_id]) ) {
		return $user_wage_obj[$user_wage_id];
	} else {
		$uwlf = new UserWageListFactory();

		//This handles future wage changes properly.
		$uwlf->getByID( $user_wage_id );
		if ( $uwlf->getRecordCount() > 0 ) {
			$user_wage_obj[$user_wage_id] = $uwlf->getCurrent();

			return $user_wage_obj[$user_wage_id];
		}

		return FALSE;
	}
}

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'action',
												'generic_data',
												'filter_data'

												) ) );

URLBuilder::setURL($_SERVER['SCRIPT_NAME'],
											array(
													'filter_data' => $filter_data
//													'sort_column' => $sort_column,
//													'sort_order' => $sort_order,
												) );

$static_columns = array(
											'-1000-full_name' => 'Full Name',
											'-1010-title' => 'Title',
											'-1020-province' => 'Province/State',
											'-1030-country' => 'Country',
											'-1039-group' => 'Group',
											'-1040-default_branch' => 'Default Branch',
											'-1050-default_department' => 'Default Department',
											'-1055-date_stamp' => 'Date',
											//'-1060-sin' => 'SIN',
											//'-1070-hire_date' => 'Hire Date',
											//'-1080-since_hire_date' => 'Since Hired'
											'-1090-hourly_rate' => 'Hourly Rate'
											);
$append_columns = array(
											'-1100-gross_wage' => 'Gross Wage',
											'-1110-paid_time' => 'Paid Time',
											'-1120-regular_time' => 'Regular Time',
											'-1130-regular_time_wage' => 'Regular Time Wage'
										);

$columns = array_merge($static_columns, $append_columns);

//Get all Overtime policies.
$otplf = new OverTimePolicyListFactory();
$otplf->getByCompanyId($current_company->getId());
if ( $otplf->getRecordCount() > 0 ) {
	foreach ($otplf as $otp_obj ) {
		Debug::Text('Over Time Policy ID: '. $otp_obj->getId() .' Rate: '. $otp_obj->getRate() , __FILE__, __LINE__, __METHOD__,10);
		$policy_rates['over_time_policy-'.$otp_obj->getId()] = $otp_obj->getRate();

		$otp_columns['over_time_policy-'.$otp_obj->getId()] = $otp_obj->getName();
		$otp_columns['over_time_policy-'.$otp_obj->getId().'_wage'] = $otp_obj->getName().' Wage';
	}

	$columns = array_merge( $columns, $otp_columns);
}

//Get all Premium policies.
$pplf = new PremiumPolicyListFactory();
$pplf->getByCompanyId($current_company->getId());
if ( $pplf->getRecordCount() > 0 ) {
	foreach ($pplf as $pp_obj ) {
		$policy_rates['premium_policy-'.$pp_obj->getId()] = $pp_obj;

		$pp_columns['premium_policy-'.$pp_obj->getId()] = $pp_obj->getName();
		$pp_columns['premium_policy-'.$pp_obj->getId().'_wage'] = $pp_obj->getName().' Wage';
	}

	$columns = array_merge( $columns, $pp_columns);
}

//Get all Absence Policies.
$aplf = new AbsencePolicyListFactory();
$aplf->getByCompanyId($current_company->getId());
if ( $aplf->getRecordCount() > 0 ) {
	foreach ($aplf as $ap_obj ) {
		$ap_columns['absence_policy-'.$ap_obj->getId()] = $ap_obj->getName();
		if ( $ap_obj->getType() == 10 ) {
			$policy_rates['absence_policy-'.$ap_obj->getId()] = 1;
			$ap_columns['absence_policy-'.$ap_obj->getId().'_wage'] = $ap_obj->getName().' Wage';
		} else {
			$policy_rates['absence_policy-'.$ap_obj->getId()] = 0;
		}
	}

	$columns = array_merge( $columns, $ap_columns);
}

$wage_columns = array('regular_time');

if ( isset( $otp_columns ) ) {
	$wage_columns = array_merge( $wage_columns, array_keys($otp_columns));
}

if ( isset( $pp_columns ) ) {
	$wage_columns = array_merge( $wage_columns, array_keys($pp_columns));
}

if ( isset( $ap_columns ) ) {
	$wage_columns = array_merge( $wage_columns, array_keys($ap_columns));
}

if ( isset($filter_data['start_date']) ) {
	$filter_data['start_date'] = TTDate::parseDateTime($filter_data['start_date']);
}

if ( isset($filter_data['end_date']) ) {
	$filter_data['end_date'] = TTDate::parseDateTime($filter_data['end_date']);
}

$filter_data = Misc::preSetArrayValues( $filter_data, array('include_user_ids', 'exclude_user_ids', 'user_status_ids', 'group_ids', 'branch_ids', 'department_ids', 'user_title_ids', 'pay_period_ids', 'columns' ), array() );

//Get Permission Hierarchy Children first, as this can be used for viewing, or editing.
$permission_children_ids = array();
if ( $permission->Check('pay_stub','view') == FALSE ) {
	$hlf = new HierarchyListFactory();
	$permission_children_ids = $hlf->getHierarchyChildrenByCompanyIdAndUserIdAndObjectTypeID( $current_company->getId(), $current_user->getId() );
	Debug::Arr($permission_children_ids,'Permission Children Ids:', __FILE__, __LINE__, __METHOD__,10);

	if ( $permission->Check('pay_stub','view_child') == FALSE ) {
		$permission_children_ids = array();
	}
	if ( $permission->Check('pay_stub','view_own') ) {
		$permission_children_ids[] = $current_user->getId();
	}
	
	$filter_data['permission_children_ids'] = $permission_children_ids;	
}

$ugdlf = new UserGenericDataListFactory();
$ugdf = new UserGenericDataFactory();

$action = strtolower($action);
switch ($action) {
	case 'export':
	case 'display report':
		//Debug::setVerbosity(11);

		Debug::Text('Submit!', __FILE__, __LINE__, __METHOD__,10);
		//Debug::Arr($filter_data, 'Filter Data', __FILE__, __LINE__, __METHOD__,10);

		$ulf = new UserListFactory();
		$ulf->getSearchByCompanyIdAndArrayCriteria( $current_company->getId(), $filter_data );
		if ( $ulf->getRecordCount() > 0 ) {
			foreach( $ulf as $u_obj ) {
				$filter_data['user_ids'][] = $u_obj->getId();
			}

			$utlf = new UserTitleListFactory();
			$title_options = $utlf->getByCompanyIdArray( $current_company->getId() );

			$uglf = new UserGroupListFactory();
			$group_options = $uglf->getArrayByNodes( FastTree::FormatArray( $uglf->getByCompanyIdArray( $current_company->getId() ), 'no_tree_text', TRUE) );

			$blf = new BranchListFactory();
			$branch_options = $blf->getByCompanyIdArray( $current_company->getId() );

			$dlf = new DepartmentListFactory();
			$department_options = $dlf->getByCompanyIdArray( $current_company->getId() );

			$udtlf = new UserDateTotalListFactory();
			$udtlf->getReportByStartDateAndEndDateAndUserIdList($filter_data['start_date'], $filter_data['end_date'], $filter_data['user_ids'] );

			foreach ($udtlf as $udt_obj ) {
				$user_id = $udt_obj->getColumn('id');

				$date_stamp = TTDate::strtotime( $udt_obj->getColumn('date_stamp') );
				$status_id = $udt_obj->getColumn('status_id');
				$type_id = $udt_obj->getColumn('type_id');
				//$user_wage_id = $udt_obj->getColumn('user_wage_id');

				if ( $status_id == 10 AND $type_id == 10 ) {
					$column = 'paid_time';
				} elseif ($status_id == 10 AND $type_id == 20) {
					$column = 'regular_time';
				} elseif ($status_id == 10 AND $type_id == 30) {
					$column = 'over_time_policy-'. $udt_obj->getColumn('over_time_policy_id');
				} elseif ($status_id == 10 AND $type_id == 40) {
					$column = 'premium_policy-'. $udt_obj->getColumn('premium_policy_id');
				} elseif ($status_id == 30 AND $type_id == 10) {
					$column = 'absence_policy-'. $udt_obj->getColumn('absence_policy_id');
				} else {
					$column = NULL;
				}

				//Debug::Text('Column: '. $column, __FILE__, __LINE__, __METHOD__,10);
				if ( ( isset($filter_data['include_no_data_users']) AND $filter_data['include_no_data_users'] == 1 )
						OR ( !isset($filter_data['include_no_data_users']) AND $date_stamp != '' AND $column != '' AND $udt_obj->getColumn('total_time') > 0 )  ) {
					$raw_rows[$user_id][$date_stamp][$column][] = array(
														'total_time' => $udt_obj->getColumn('total_time'),
														'user_wage_id' => $udt_obj->getColumn('user_wage_id')
														);
				}
			}
			//var_dump($raw_rows);

			$ulf = new UserListFactory();

			if ( isset($raw_rows) ) {
				$x=0;

				foreach($raw_rows as $user_id => $data_a ) {

					foreach($data_a as $date_stamp => $data_b) {
						$user_obj = $ulf->getById( $user_id )->getCurrent();

						$tmp_rows[$x]['user_id'] = $user_id;
						$tmp_rows[$x]['full_name'] = $user_obj->getFullName(TRUE);
						$tmp_rows[$x]['province'] = $user_obj->getProvince();
						$tmp_rows[$x]['country'] = $user_obj->getCountry();
						
						$tmp_rows[$x]['title'] = Option::getByKey($user_obj->getTitle(), $title_options, NULL );
						$tmp_rows[$x]['group'] = Option::getByKey($user_obj->getGroup(), $group_options );
						$tmp_rows[$x]['default_branch'] =  Option::getByKey($user_obj->getDefaultBranch(), $branch_options, NULL );
						$tmp_rows[$x]['default_department'] = Option::getByKey($user_obj->getDefaultDepartment(), $department_options, NULL );

/*						
						$tmp_rows[$x]['title'] = $title_options[$user_obj->getTitle()];
						if ( isset($branch_options[$user_obj->getDefaultBranch()]) ) {
							$tmp_rows[$x]['default_branch'] = $branch_options[$user_obj->getDefaultBranch()];
						} else {
							$tmp_rows[$x]['default_branch'] = NULL;
						}
						if ( isset($department_options[$user_obj->getDefaultDepartment()]) ) {
							$tmp_rows[$x]['default_department'] = $department_options[$user_obj->getDefaultDepartment()];
						} else {
							$tmp_rows[$x]['default_department'] = NULL;
						}
*/
						$tmp_rows[$x]['date_stamp'] = $date_stamp;

						foreach($data_b as $column => $data_c) {
							foreach($data_c as $data_d) {
								$total_time = $data_d['total_time'];
								$user_wage_id = $data_d['user_wage_id'];

								$hourly_rate = 0;
								/*
								if ( isset($user_wage_obj[$user_wage_id]) AND is_object($user_wage_obj[$user_wage_id]) ) {
									$hourly_rate = $user_wage_obj[$user_wage_id]->getHourlyRate();
								} else {
									$uw_obj = getUserWageObject($user_wage_id);
									if ( is_object($uw_obj) ) {
										$hourly_rate = $uw_obj->getHourlyRate();
									}
								}
								*/
								$uw_obj = getUserWageObject($user_wage_id);
								if ( is_object($uw_obj) ) {
									$hourly_rate = $uw_obj->getHourlyRate();
								}
								$tmp_rows[$x]['hourly_rate'] = Misc::MoneyFormat( $hourly_rate, FALSE );

								if ( isset($tmp_rows[$x][$column]) ) {
									$tmp_rows[$x][$column] += $total_time;
								} else {
									$tmp_rows[$x][$column] = $total_time;
								}

								//Get Rate
								if ( isset($policy_rates[$column]) ) {
									if ( strpos( $column, 'premium') !== FALSE AND is_object($policy_rates[$column]) ) {
										Debug::Text('Column: '. $column .' Premium Policy, has dynamic rate.', __FILE__, __LINE__, __METHOD__,10);
										$policy_hourly_rate = $policy_rates[$column]->getHourlyRate( $hourly_rate );
									} else {
										Debug::Text('Column: '. $column .' NOT Premium Policy: Policy Rate: '. $policy_rates[$column], __FILE__, __LINE__, __METHOD__,10);
										$policy_hourly_rate = $policy_rates[$column] * $hourly_rate;
									}
								} elseif ($column == 'regular_time') {
									//$rate = 1;
									$policy_hourly_rate = $hourly_rate;
								} else {
									$policy_hourly_rate = 0;
									//$rate = 0;
								}
								//Debug::Text('Column: '. $column .' Total Time: '. $total_time .' Hourly Rate: '. $hourly_rate .' Policy Hourly Rate: '. $policy_hourly_rate .' Amount: '. TTDate::getHours( $total_time ) * $policy_hourly_rate, __FILE__, __LINE__, __METHOD__,10);

								$amount = TTDate::getHours( $total_time ) * $policy_hourly_rate;

								if ( in_array($column, $wage_columns) ) {
									//Debug::Text('...Column: '. $column .' Hourly Wage: '. $hourly_rate .' Rate: '. $rate .' Total Wage: '. ($hourly_rate * $rate), __FILE__, __LINE__, __METHOD__,10);

									if ( isset($tmp_rows[$x][$column.'_wage'] ) ) {
										$tmp_rows[$x][$column.'_wage'] = Misc::MoneyFormat( $tmp_rows[$x][$column.'_wage'] + $amount, FALSE);
									} else {
										$tmp_rows[$x][$column.'_wage'] = Misc::MoneyFormat( $amount, FALSE);
									}
								}

								if ( isset($tmp_rows[$x]['gross_wage'] ) ) {
									//Debug::Text('Adding to Gross, Prev Amount: '. $tmp_rows[$x]['gross_wage'] .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
									$tmp_rows[$x]['gross_wage'] = Misc::MoneyFormat( $tmp_rows[$x]['gross_wage'] + $amount, FALSE);
								} else {
									//Debug::Text('Setting Gross...', __FILE__, __LINE__, __METHOD__,10);
									$tmp_rows[$x]['gross_wage'] = Misc::MoneyFormat( $amount, FALSE);
								}

								unset($hourly_rate, $policy_hourly_rate, $total_time);

							}
						}

						$x++;
					}

				}
			}
			//var_dump($tmp_rows);

			if ( isset($tmp_rows) AND isset($filter_data['primary_group_by']) AND $filter_data['primary_group_by'] != '0' ) {
				Debug::Text('Primary Grouping Data By: '. $filter_data['primary_group_by'], __FILE__, __LINE__, __METHOD__,10);

				$ignore_elements = array_keys($static_columns);

				$filter_data['columns'] = array_diff( $filter_data['columns'], $ignore_elements );

				//Add the group by element back in
				if ( isset($filter_data['secondary_group_by']) AND $filter_data['secondary_group_by'] != 0 ) {
					array_unshift( $filter_data['columns'], $filter_data['primary_group_by'], $filter_data['secondary_group_by'] );
				} else {
					array_unshift( $filter_data['columns'], $filter_data['primary_group_by'] );
				}

				$tmp_rows = Misc::ArrayGroupBy( $tmp_rows, array(Misc::trimSortPrefix($filter_data['primary_group_by']),Misc::trimSortPrefix($filter_data['secondary_group_by'])), Misc::trimSortPrefix($ignore_elements) );
			}

			if ( isset($tmp_rows) ) {
				foreach($tmp_rows as $row) {
					$rows[] = $row;
				}

				$rows = Sort::Multisort($rows, Misc::trimSortPrefix($filter_data['primary_sort']), Misc::trimSortPrefix($filter_data['secondary_sort']), $filter_data['primary_sort_dir'], $filter_data['secondary_sort_dir']);

				$total_row = Misc::ArrayAssocSum($rows, NULL, 2);

				$last_row = count($rows);
				$rows[$last_row] = $total_row;
				foreach ($static_columns as $static_column_key => $static_column_val) {
					$rows[$last_row][Misc::trimSortPrefix($static_column_key)] = NULL;
				}
				unset($static_column_key, $static_column_val);

				$tmp_rows = $rows;
				unset($rows);
			}

			if ( isset($tmp_rows) ) {
				$trimmed_static_columns = array_keys( Misc::trimSortPrefix($static_columns) );
				foreach($tmp_rows as $tmp_row ) {
					foreach($tmp_row as $column => $column_data) {
						//if ( strstr($column, 'time') AND !strstr($column, 'wage') AND $column_data != '' ) {
						if ( $column == 'date_stamp') {
							$column_data = TTDate::getDate('DATE', $column_data);
						} elseif ( !in_array( $column, $trimmed_static_columns )
								AND !strstr($column, 'wage') AND $column_data != '' ) {
							//Debug::Text('Converting to Time Column: '. $column, __FILE__, __LINE__, __METHOD__,10);
							$column_data = TTDate::getTimeUnit( $column_data );
						}

						$row_columns[$column] = $column_data;
						unset($column, $column_data);
					}

					$rows[] = $row_columns;
					unset($row_columns);
				}
			}

			//var_dump($rows);
			foreach( $filter_data['columns'] as $column_key ) {
				$filter_columns[Misc::trimSortPrefix($column_key)] = $columns[$column_key];
			}
		}

		if ( $action == 'export' ) {
			if ( isset($rows) AND isset($filter_columns) ) {
				Debug::Text('Exporting as CSV', __FILE__, __LINE__, __METHOD__,10);
				$data = Misc::Array2CSV( $rows, $filter_columns );

				Misc::FileDownloadHeader('report.csv', 'application/csv', strlen($data) );
				echo $data;
			} else {
				echo "No Data To Export!<br>\n";
			}
		} else {
			$smarty->assign_by_ref('generated_time', TTDate::getTime() );
			$smarty->assign_by_ref('filter_data', $filter_data );
			$smarty->assign_by_ref('columns', $filter_columns );
			$smarty->assign_by_ref('rows', $rows);

			$smarty->display('report/WagesPayableSummaryReport.tpl');
		}

		break;
	case 'delete':
	case 'save':
		Debug::Text('Action: '. $action, __FILE__, __LINE__, __METHOD__,10);

		$generic_data['id'] = UserGenericDataFactory::reportFormDataHandler( $action, $filter_data, $generic_data, URLBuilder::getURL(NULL, $_SERVER['SCRIPT_NAME']) );
		unset($generic_data['name']);
	default:
		BreadCrumb::setCrumb($title);

		if ( $action == 'load' ) {
			Debug::Text('Loading Report!', __FILE__, __LINE__, __METHOD__,10);
			extract( UserGenericDataFactory::getReportFormData( $generic_data['id'] ) );
		} elseif ( $action == '' ) {
			//Check for default saved report first.
			$ugdlf->getByUserIdAndScriptAndDefault( $current_user->getId(), $_SERVER['SCRIPT_NAME'] );
			if ( $ugdlf->getRecordCount() > 0 ) {
				Debug::Text('Found Default Report!', __FILE__, __LINE__, __METHOD__,10);

				$ugd_obj = $ugdlf->getCurrent();
				$filter_data = $ugd_obj->getData();
				$generic_data['id'] = $ugd_obj->getId();
			} else {
				Debug::Text('Default Settings!', __FILE__, __LINE__, __METHOD__,10);
				//Default selections

				$filter_data['user_status_ids'] = array( -1 );
				$filter_data['branch_ids'] = array( -1 );
				$filter_data['department_ids'] = array( -1 );
				$filter_data['user_title_ids'] = array( -1 );
				$filter_data['group_ids'] = array( -1 );

				$filter_data['columns'] = array(
											'-1000-full_name',
											'-1100-gross_wage',
											'-1110-paid_time',
											'-1120-regular_time'
												);

				$filter_data['start_date'] = TTDate::getBeginMonthEpoch();
				$filter_data['end_date'] = TTDate::getEndMonthEpoch();

				$filter_data['primary_group_by'] = '-1000-full_name';

				$filter_data['primary_sort'] = '-1000-full_name';
				$filter_data['secondary_sort'] = '-1100-gross_wage';
			}
		}		
		$filter_data = Misc::preSetArrayValues( $filter_data, array('include_user_ids', 'exclude_user_ids', 'user_status_ids', 'group_ids', 'branch_ids', 'department_ids', 'user_title_ids', 'pay_period_ids', 'columns' ), NULL);

		$ulf = new UserListFactory();
		$all_array_option = array('-1' => '-- All --');

		//Get include employee list.
		$ulf->getSearchByCompanyIdAndArrayCriteria( $current_company->getId(), array('permission_children_ids' => $permission_children_ids ) );
		$user_options = $ulf->getArrayByListFactory( $ulf, FALSE, TRUE );
		$filter_data['src_include_user_options'] = Misc::arrayDiffByKey( (array)$filter_data['include_user_ids'], $user_options );
		$filter_data['selected_include_user_options'] = Misc::arrayIntersectByKey( (array)$filter_data['include_user_ids'], $user_options );

		//Get exclude employee list
		$exclude_user_options = Misc::prependArray( $all_array_option, $ulf->getArrayByListFactory( $ulf, FALSE, TRUE ) );
		$filter_data['src_exclude_user_options'] = Misc::arrayDiffByKey( (array)$filter_data['exclude_user_ids'], $user_options );
		$filter_data['selected_exclude_user_options'] = Misc::arrayIntersectByKey( (array)$filter_data['exclude_user_ids'], $user_options );

		//Get employee status list.
		$user_status_options = Misc::prependArray( $all_array_option, $ulf->getOptions('status') );
		$filter_data['src_user_status_options'] = Misc::arrayDiffByKey( (array)$filter_data['user_status_ids'], $user_status_options );
		$filter_data['selected_user_status_options'] = Misc::arrayIntersectByKey( (array)$filter_data['user_status_ids'], $user_status_options );

		//Get Employee Groups
		$uglf = new UserGroupListFactory();
		$group_options = Misc::prependArray( $all_array_option, $uglf->getArrayByNodes( FastTree::FormatArray( $uglf->getByCompanyIdArray( $current_company->getId() ), 'TEXT', TRUE) ) );
		$filter_data['src_group_options'] = Misc::arrayDiffByKey( (array)$filter_data['group_ids'], $group_options );
		$filter_data['selected_group_options'] = Misc::arrayIntersectByKey( (array)$filter_data['group_ids'], $group_options );

		//Get branches
		$blf = new BranchListFactory();
		$blf->getByCompanyId( $current_company->getId() );
		$branch_options = Misc::prependArray( $all_array_option, $blf->getArrayByListFactory( $blf, FALSE, TRUE ) );
		$filter_data['src_branch_options'] = Misc::arrayDiffByKey( (array)$filter_data['branch_ids'], $branch_options );
		$filter_data['selected_branch_options'] = Misc::arrayIntersectByKey( (array)$filter_data['branch_ids'], $branch_options );

		//Get departments
		$dlf = new DepartmentListFactory();
		$dlf->getByCompanyId( $current_company->getId() );
		$department_options = Misc::prependArray( $all_array_option, $dlf->getArrayByListFactory( $dlf, FALSE, TRUE ) );
		$filter_data['src_department_options'] = Misc::arrayDiffByKey( (array)$filter_data['department_ids'], $department_options );
		$filter_data['selected_department_options'] = Misc::arrayIntersectByKey( (array)$filter_data['department_ids'], $department_options );

		//Get employee titles
		$utlf = new UserTitleListFactory();
		$utlf->getByCompanyId( $current_company->getId() );
		$user_title_options = Misc::prependArray( $all_array_option, $utlf->getArrayByListFactory( $utlf, FALSE, TRUE ) );		
		$filter_data['src_user_title_options'] = Misc::arrayDiffByKey( (array)$filter_data['user_title_ids'], $user_title_options );
		$filter_data['selected_user_title_options'] = Misc::arrayIntersectByKey( (array)$filter_data['user_title_ids'], $user_title_options );

		//Get column list
		$filter_data['src_column_options'] = Misc::arrayDiffByKey( (array)$filter_data['columns'], $columns );
		$filter_data['selected_column_options'] = Misc::arrayIntersectByKey( (array)$filter_data['columns'], $columns );

		//Get primary/secondary order list
		$filter_data['sort_options'] = $columns;
		$filter_data['sort_options']['effective_date_order'] = 'Wage Effective Date';
		unset($filter_data['sort_options']['effective_date']);
		$filter_data['sort_direction_options'] = Misc::getSortDirectionArray();

		$filter_data['group_by_options'] = Misc::prependArray( array('0' => 'No Grouping'), $static_columns );

		$saved_report_options = $ugdlf->getByUserIdAndScriptArray( $current_user->getId(), $_SERVER['SCRIPT_NAME']);
		$generic_data['saved_report_options'] = $saved_report_options;
		$smarty->assign_by_ref('generic_data', $generic_data);

		$smarty->assign_by_ref('filter_data', $filter_data);

		$smarty->assign_by_ref('ugdf', $ugdf);

		$smarty->display('report/WagesPayableSummary.tpl');

		break;
}
?>