<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 514 $
 * $Id: ViewSchedule.php 514 2007-01-10 00:59:00Z ipso $
 * $Date: 2007-01-09 16:59:00 -0800 (Tue, 09 Jan 2007) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');
require_once(Environment::getBasePath() .'classes/misc/arr_multisort.class.php');

//Debug::setVerbosity(11);

if ( !$permission->Check('schedule','enabled')
		OR !( $permission->Check('schedule','view') OR $permission->Check('schedule','view_own') OR $permission->Check('schedule','view_child') ) ) {
	$permission->Redirect( FALSE ); //Redirect
}

$title = 'My Schedule';
$smarty->assign('title', $title);
BreadCrumb::setCrumb($title);

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'do',
												'page',
												'sort_column',
												'sort_order',
												'filter_view_type_id',
												'filter_start_date',
												'filter_show_days',
												'filter_user_id'
												) ) );

URLBuilder::setURL($_SERVER['SCRIPT_NAME'],
											array(
													'sort_column' => $sort_column,
													'sort_order' => $sort_order,
													'page' => $page
												) );
if ( $filter_start_date != '' ) {
	$filter_start_date = TTDate::strtotime($filter_start_date);
}

//Get Permission Hierarchy Children first, as this can be used for viewing, or editing.
$hlf = new HierarchyListFactory();
$permission_children_ids = $hlf->getHierarchyChildrenByCompanyIdAndUserIdAndObjectTypeID( $current_company->getId(), $current_user->getId() );
Debug::Arr($permission_children_ids,'Permission Children Ids:', __FILE__, __LINE__, __METHOD__,10);
$filter_data = NULL;
if ( $permission->Check('schedule','view_child') ) {
	$filter_data['permission_children_ids'] = $permission_children_ids;
}
if ( $permission->Check('schedule','view') == FALSE AND $permission->Check('schedule','view_own') == TRUE ) {
	$filter_data['permission_children_ids'][] = $current_user->getId();
}


switch (strtolower($do)) {
	case 'filter':
		if ( $filter_start_date != '' AND $filter_show_days != '' ) {
			$start_date = $filter_start_date = TTDate::getBeginDayEpoch( $filter_start_date );
			$end_date = $start_date + ($filter_show_days*86400-3600);
		}
	default:
		if ( $filter_user_id == '' ) {
			$filter_user_id = array($current_user->getId());
		}
		if ( $filter_show_days == '' ) {
			$filter_show_days = 4;
		}

		if ( $filter_start_date == '' OR $filter_show_days == '' ) {
			$start_date = $filter_start_date = TTDate::getBeginWeekEpoch( TTDate::getTime(), $current_user_prefs->getStartWeekDay() );
			//$end_date = $filter_show_days = $start_date + (7*(86400-3600));
		}

		$smarty->assign_by_ref('filter_view_type_id', $filter_view_type_id);
		$smarty->assign_by_ref('filter_start_date', $filter_start_date);
		$smarty->assign_by_ref('filter_show_days', $filter_show_days);

		$ulf = new UserListFactory();
		$ulf->getSearchByCompanyIdAndArrayCriteria( $current_company->getId(), $filter_data );
		$user_options = UserListFactory::getArrayByListFactory( $ulf, FALSE, FALSE );
		//$user_options = UserListFactory::getByCompanyIdArray( $current_company->getId(), FALSE, FALSE );
		$smarty->assign_by_ref('user_options', $user_options);

		if ( $filter_user_id != '' AND is_array($filter_user_id) ) {
			$tmp_user_options = UserListFactory::getByCompanyIdArray( $current_company->getId(), FALSE, TRUE );
			foreach( $filter_user_id as $user_id ) {
				$filter_user_options[$user_id] = $tmp_user_options[$user_id];
			}
		}
		$smarty->assign_by_ref('filter_user_options', $filter_user_options);

		//$show_days_options = array(7 => 7, 14 => 14, 21 => 21, 28 => 28, 35 => 35, 42 => 42, 49 => 49, 56 => 56, 63 => 63);
		$show_days_options = array(1 => '1 Week', 2 => '2 Weeks', 3 => '3 Weeks', 4 => '4 Weeks', 5 => '5 Weeks', 6 => '6 Weeks');
		$smarty->assign_by_ref('show_days_options', $show_days_options );

		$view_type_options = array( 10 => 'Month', 20 => 'Week', 30 => 'Day' );
		$smarty->assign_by_ref('view_type_options', $view_type_options );

		$smarty->assign_by_ref('filter_user_id', $filter_user_id);

		break;
}
$smarty->display('schedule/ViewSchedule.tpl');
?>